
{% call widget.panel(_("Change Password")) %}

    {% call forms.post(request.endpoint) %}

        {{ forms.input(label="Type New Password",
            name="password", type="password",
            data={"validation": "required"}) }}

        {{ forms.input(label="Re-Type New Password",
            name="password2", type="password",
            data={"validation": "required"}) }}

            {{ forms.input(label="Confirm your change by entering your current password",
            name="confirm-password", type="password",
            data={"validation": "required"}) }}

        {{ forms.button('Change Password', type='submit', class='btn-info') }}
    {% endcall %}
{% endcall %}