
    {% call widget.panel('Reset Password') %}

        {{ widget.alert("You need to reset your password to continue.", "warning", false) }}

        {% set action = url_for(request.endpoint, token=token) %}
        {% call forms.post(action=action) %}

            {{ forms.input(label="New Password", name="password", type="password",
                data={"validation": "required",
                      "validation-error-msg": "Password is required"}) }}

            {{ forms.input(label="Confirm Password", name="password2", type="password",
                data={"validation":"required",
                      "validation-confirm":"password",
                      "validation-error-msg": "Password is required"}) }}

            {{ forms.button("Save New Password", type="submit", class="btn btn-primary btn-lg btn-block") }}
        {% endcall %}

    {% endcall %}

