

{% call widget.panel(_("Account Info"), type="table") %}

    <table class="table table-bordered">
        <tbody>
            <tr>
                <td>Email</td>
                <td>{{ current_user.email }} [ <a href="{{ url_for('AuthAccount:change_email') }}">Change Email</a>]</td>
            </tr>
            <tr>
                <td>Name</td>
                <td>{{ current_user.name or ""}}</td>
            </tr>
            <tr>
                <td>First Name</td>
                <td>{{ current_user.first_name or ""}}</td>
            </tr>
            <tr>
                <td>Last Name</td>
                <td>{{ current_user.last_name  or "" }}</td>
            </tr>
            <tr>
                <td>DOB</td>
                <td>{{ current_user.dob or "" }}</td>
            </tr>
            <tr>
                <td>Address</td>
                <td>{{ current_user.address or "" }}</td>
            </tr>
            <tr>
                <td>State</td>
                <td>{{ current_user.address_state or "" }}</td>
            </tr>
            <tr>
                <td>City</td>
                <td>{{ current_user.address_city or "" }}</td>
            </tr>
            <tr>
                <td>Zip Code</td>
                <td>{{ current_user.address_zip_code or "" }}</td>
            </tr>
            <tr>
                <td>Country</td>
                <td>{{ current_user.address_country or "" }}</td>
            </tr>
            <tr>
                <td>Telephhone</td>
                <td>{{ current_user.telephone or "" }}</td>
            </tr>
        </tbody>

    </table>

{% endcall %}

<a class="btn btn-info" href="{{ url_for('AuthAccount:edit_account_info') }}">Change Account Info</a>