
@import "mixins";

// Global namespace: mambo-*


.responsive-image {
    @include responsive-image;
    border-radius: 4px;
}

.navbar-fixed-top {

}

.image-circle {
  border-radius: 50%;
}

// img.profile-avatar
img {
  &.mambo-profile-avatar {
      @include profile-avatar(100px);
    &-30{
      @include profile-avatar(30px);
    }
    &-40{
      @include profile-avatar(40px);
    }
    &-50{
      @include profile-avatar(50px);
    }
    &-100{
      @include profile-avatar(100px);
    }
    &-150{
      @include profile-avatar(150px);
    }
    &-200{
      @include profile-avatar(200px);
    }
    &-250{
      @include profile-avatar(250px);
    }
  }
}

.mambo {
  &-share-buttons {
    font-size: 20px;
    a {
      text-decoration: none !important;
    }
  }
}






