// _mixin.scss

$font : "Lato", sans-serif;

@mixin clearfix {
    &:before,
    &:after {
        content: " ";
        display: table;
    }
    &:after {
        clear: both
    }
}

// Responsive image
@mixin responsive-image {
    img {
        display: inline-block;
        height: auto;
        max-width: 100%;
    }
}


