
    {% call widget.panel( login_title or 'Signup') %}

        {% if allow_signup and oauth_enabled %}
            <div class="text-center">
                {{ macros.oauth_providers() }}
            </div>

            <hr>
        {% endif %}

        {% call forms.post(request.endpoint) %}

            {{ forms.hidden("method", value="email") }}
            {{ forms.hidden("next", value=login_url_next) }}

            {{ forms.input(label="Name", name="name",
                data={"validation": "required",
                      "validation-error-msg": "Name is required"}) }}

            {{ forms.input(label="Email", name="email", type="email", data={"validation": "email"}) }}

            {{ forms.input(label="Password", name="password", type="password",
                data={"validation": "required",
                      "validation-error-msg": "Password is required"}) }}

            {{ forms.input(label="Confirm Password", name="password2", type="password",
                data={"validation":"required",
                      "validation-confirm":"password",
                      "validation-error-msg": "Password is required"}) }}

            {% call forms.group() %}
                {{ recaptcha }}
            {% endcall %}

            {{ forms.button("Signup", type="submit", class="btn-primary btn-lg btn-block") }}
        {% endcall %}

        <hr>

        <a href="{{url_for('AuthLogin:login', next=login_url_next)}}"><i class="fa fa-arrow-left"></i> Login</a>

    {% endcall %}










